#include "ProbitML.h"
#include "ProbitRegression.h"

using namespace std;

void ProbitRegression(DataTable TheTable) {

	int j, nobs;
	valarray<double> x, N, r;	// want x, N, and r as DblLists for convenience in loglikelihood
	int npar = 2;		// hard-coded two-parameter problem (a bit of a kluge)

	nobs = TheTable.N;
	x.resize(nobs);		// this section changes the size of the DblLists
	N.resize(nobs);		// and then copies the columns of the DataTable there
	r.resize(nobs);
	for (j = 0; j < nobs; j++)  {
		x[j] = TheTable.Data(j+1,1);
		N[j] = TheTable.Data(j+1,2);
		r[j] = TheTable.Data(j+1,3);
	}

	ColumnVector p(npar);		// both p
	ColumnVector para(npar);	// and para are copies of the parameter vector
	SymmetricMatrix Hinv(npar);	// will be the Information matrix here
	Real startvalues[] = {0.0, 5.0};	// starting values for N-R maximization
	p << startvalues;					// now loaded into '
	
	ProbitML_LL ProbitML(x, N, r, p, nobs); // create loglikehood "object" ProbitML

	MLE_D_FI mle_d_fi(ProbitML,20,0.0000001);    // another "object" of type MLE_D_FI

	para = p;			// anothe copy of p; have never been quite sure why
	mle_d_fi.Fit(para);	// do the fit, routein  in MLE_D_FI in NEWMATNL

	cout << setprecision(4) << resetiosflags(ios::right) << endl;
	cout << "Estimated parameters:" << endl << para << endl;
	Hinv = ProbitML.FI().i();
	cout << setprecision(6);
	cout << "Inverse Hessian:" << endl << Hinv << endl;
	cout << "Standard Errors:" << setprecision(4) << endl;
	for (int i=1; i <= npar; i++) cout << sqrt(Hinv(i,i)) << endl;
	
}

// below are the NormalProb routines
double NormalProb(double z) {
	double x,t,p;
	
	if(z < 0) x = -z; else x = z;
	
	t = 1/(1+0.2316419*x);
	p = ((((1.330274429*t-1.821255978)*t+1.781477937)*t-0.356563782)*t+0.319381530)*t;
	p = 1.0 - 0.398942280401*exp(-0.5*x*x)*p;
	
	if(z < 0) p = 1 - p;
	return p;
}

valarray<double> NormalProbList(valarray<double> zList, int size) {
	valarray<double> pList(size);
	for (int i = 0; i < size; i++) {
		pList[i] = NormalProb(zList[i]);
	}
	return pList;
}
